/*
 *                 Sun Public License Notice
 *
 * The contents of this file are subject to the Sun Public License
 * Version 1.0 (the "License"). You may not use this file except in
 * compliance with the License. A copy of the License is available at
 * http://www.sun.com/
 *
 *
 * The Original Code is NetBeans. The Initial Developer of the Original
 * Code is Tom Wheeler. Portions Copyright 2006 Tom Wheeler. All Rights 
 * Reserved. Any future updates to this code will be made available at 
 * http://www.tomwheeler.com/netbeans/
 */

package com.tomwheeler.example.tdlfile.gui;

import com.tomwheeler.example.tdlfile.intf.TodoListItem;
import org.openide.nodes.Node;
import org.openide.util.HelpCtx;
import org.openide.util.Lookup;
import org.openide.util.NbBundle;
import org.openide.util.actions.NodeAction;

/**
 * <p>
 * Documentation still TODO; in the meantime, the commentary in AddItemAction 
 * applies here.
 * </p>
 * 
 * @author <a href="http://www.tomwheeler.com/netbeans/">Tom Wheeler</a>
 */
public final class EditSingleItemAction extends NodeAction {
    
    /** the main business logic of this class */
    protected void performAction(Node[] activatedNodes) {
        Node firstNode = activatedNodes[0];
        
        Lookup nodeLookup = firstNode.getLookup();
        TodoListItem item = (TodoListItem) nodeLookup.lookup(TodoListItem.class);
        if (item == null) {
            System.err.println("Selected node's lookup does not contain a TodoListItem");
            return;
        }
        
        TodoListItemEditorTopComponent tc = TodoListItemEditorTopComponent.findInstance();
        tc.open();
        tc.requestActive();
        
        tc.setActivatedNodes(activatedNodes); // may not be necessary
        tc.setTodoListItem(item);        
    }
    
    public String getName() {
        return NbBundle.getMessage(EditSingleItemAction.class, "CTL_EditSingleItemAction");
    }
    
    protected String iconResource() {
        return "com/tomwheeler/example/tdlfile/resources/tdlitem.png";
    }
    
    public HelpCtx getHelpCtx() {
        return HelpCtx.DEFAULT_HELP;
    }
    
    protected boolean asynchronous() {
        return false;
    }
    
    protected boolean enable(Node[] activatedNodes) {
        // this action is enabled only when exactly one node is selected
        if (activatedNodes == null || activatedNodes.length != 1) {
            return false;
        }
        
        // and that node has a TodoListItem we can edit
        Lookup nodeLookup = activatedNodes[0].getLookup();
        return nodeLookup.lookup(TodoListItem.class) != null;
    }
}

