/*
 *                 Sun Public License Notice
 *
 * The contents of this file are subject to the Sun Public License
 * Version 1.0 (the "License"). You may not use this file except in
 * compliance with the License. A copy of the License is available at
 * http://www.sun.com/
 *
 *
 * The Original Code is NetBeans. The Initial Developer of the Original
 * Code is Tom Wheeler. Portions Copyright 2006 Tom Wheeler. All Rights 
 * Reserved. Any future updates to this code will be made available at 
 * http://www.tomwheeler.com/netbeans/
 */

package com.tomwheeler.example.tdlfile.impl;

import com.tomwheeler.example.tdlfile.intf.*;
import java.beans.PropertyChangeListener;
import java.beans.PropertyChangeSupport;
import java.io.IOException;
import java.io.ObjectInputStream;
import java.util.Date;
import java.util.Iterator;

/**
 * <p>
 * Basic implementation of the TodoListItem interface; it models a single item 
 * on the TODO list using standard JavaBeans conventions.
 * </p>
 *
 * @author <a href="http://www.tomwheeler.com/netbeans/">Tom Wheeler</a>
 */
public class BasicTodoListItem implements TodoListItem {

    private static final long serialVersionUID = 1L;
    
    private BasicTodoList parent;
    private String name, desc;
    private Date dueDate;
    // TODO add priority field?  (enum: low, medium, high)
    
    private transient PropertyChangeSupport propChangeSupport;
    
    /** Creates a new instance of TodoListItem using all default values */
    public BasicTodoListItem(BasicTodoList parent) {
        this(parent, "ChangeMe", "ChangeMe", new Date());
    }

    /** Creates a new instance of TodoListItem using the specified name, desc and due date */
    public BasicTodoListItem(BasicTodoList parent, String name, String desc, Date dueDate) {
        propChangeSupport = new PropertyChangeSupport(this);

        this.parent = parent;
        setName(name);
        setDesc(desc);
        setDueDate(dueDate);        
    }
    
    public TodoList getTodoList() {
        return parent;
    }

    
    public String getName() {
        return name;
    }
    
    public void setName(String newValue) {
        assert(newValue != null);

        String oldValue = name;
        this.name = newValue;
        propChangeSupport.firePropertyChange(PROP_NAME, oldValue, newValue);
    }
    
    public String getDesc() {
        return desc;
    }
    
    public void setDesc(String newValue) {
        String oldValue = desc;
        this.desc = newValue;
        propChangeSupport.firePropertyChange(PROP_DESC, oldValue, newValue);
    }
    
    public Date getDueDate() {
        return dueDate;
    }
    
    public void setDueDate(Date newValue) {
        Date oldValue = dueDate;
        this.dueDate = newValue;
        propChangeSupport.firePropertyChange(PROP_DUE_DATE, oldValue, newValue);
    }
    
    public synchronized void addPropertyChangeListener(PropertyChangeListener listener) {
        propChangeSupport.addPropertyChangeListener(listener);
    }
    
    public synchronized void removePropertyChangeListener(PropertyChangeListener listener) {
        propChangeSupport.removePropertyChangeListener(listener);
    }
    
    public boolean equals(Object o) {
        // name is is the only significant identifier field.
        if (this == o) {
            return true;
        }

        if (! (o instanceof BasicTodoListItem)) {
            return false;
        }
        
        BasicTodoListItem that = (BasicTodoListItem)o;
        return name == null ? that.name != null : ! name.equals(that.name);
    }
    
    public int hashCode() {
        return name == null ? 0 : name.hashCode();
    }
    
    public String toString() {
        return "BasicTodoListItem: [ name= " + name + ", desc=" + desc + ", dueDate=" + dueDate + " ]";
    }

    /**
     * <p>
     * It's necessary to implement this because deserialization does not use the
     * constructor and we have to ensure that the property change support
     * instance gets initialized.
     * </p>
     *
     * @param in the input stream
     * @throws IOException
     * @throws ClassNotFoundException
     */
    private void readObject(ObjectInputStream in) throws IOException, ClassNotFoundException {
        if (propChangeSupport == null) {
            propChangeSupport = new PropertyChangeSupport(this);
        }

        in.defaultReadObject();
    }
}
