/*
 *                 Sun Public License Notice
 *
 * The contents of this file are subject to the Sun Public License
 * Version 1.0 (the "License"). You may not use this file except in
 * compliance with the License. A copy of the License is available at
 * http://www.sun.com/
 *
 *
 * The Original Code is NetBeans. The Initial Developer of the Original
 * Code is Tom Wheeler. Portions Copyright 2006 Tom Wheeler. All Rights 
 * Reserved. Any future updates to this code will be made available at 
 * http://www.tomwheeler.com/netbeans/
 */

package com.tomwheeler.example.tdlfile.intf;

import java.beans.PropertyChangeListener;
import java.io.Serializable;
import java.util.Date;

/**
 * <p>
 * Interface which defines the minimal functionality for a single item in the 
 * TODO list.  The name property is the only significant identifier; names should
 * not be duplicated in any given list.
 * </p>
 *
 * @author <a href="http://www.tomwheeler.com/netbeans/">Tom Wheeler</a>
 */
public interface TodoListItem extends Serializable {

    /** constant representing the <code>desc</code> property */
    String PROP_DESC = "desc";

    /** constant representing the <code>dueDate</code> property */
    String PROP_DUE_DATE = "dueDate";
    
    /** constant representing the <code>name</code> property */
    String PROP_NAME = "name";

    /** returns the list to which this item belongs */
    TodoList getTodoList();

    /** returns the name (display label) for this TodoListItem */
    String getName();

    /**  sets the name (display label) of this TodoListItem.  */
    void setName(String newValue);

    /** returns the description of this TodoListItem */
    String getDesc();

    /** sets the description of this TodoListItem */
    void setDesc(String newValue);

    /** returns the date on which this TodoListItem is due */
    Date getDueDate();

    /** sets the date on which this TodoListItem is due */
    void setDueDate(Date newValue);

    /** adds a property change listener which is notified about changes to this instance's values */
    void addPropertyChangeListener(PropertyChangeListener listener);
    
    /** removes a property change listener that is listening to changes on this instance's values */
    void removePropertyChangeListener(PropertyChangeListener listener);
    
}
